import type { EmitterSubscription, EventSubscription  } from "react-native";
import Sensor from "../Sensor";
import type { EventNeuroSmartMEMSData, SignalChannelsData, ResistRefChannelsData, BrainBit2AmplifierParam, EventFPGData,
    EEGChannelInfo, MEMSData, SensorAmpMode, FPGData, SensorSamplingFrequency, IrAmplitude, RedAmplitude, EventBrainBit2SignalData,
    SensorAccelerometerSensitivity, SensorGyroscopeSensitivity, EventAmpModeChangedData, EventBrainBit2ResistData } from "../NativeNeurosdk2";
import { Neurosdk2 } from "../ModuleResolver";

export default class BrainBit2Sensor extends Sensor {

    private resistDataReceived: EmitterSubscription | undefined;
    private newResistDataReceived: EventSubscription | undefined;
    private signalDataReceived: EmitterSubscription | undefined;
    private newSignalDataReceived: EventSubscription | undefined;

    private AMPModeChanged: EmitterSubscription | undefined;
    private newAMPModeChanged: EventSubscription | undefined;
    private MEMSDataChanged: EmitterSubscription | undefined;
    private newMEMSDataChanged: EventSubscription | undefined;
    private neuroSmartFPGChanged: EmitterSubscription | undefined;
    private newNeuroSmartFPGChanged: EventSubscription | undefined;

    constructor(guid: string) {
        super(guid)
    }

    static getMaxChCount(): number{
        return 8;
    }

    enable66HzFilter(): void{
        Neurosdk2.set66HzFEnabled(this.guid, true);
    }

    disable66HzFilter(): void{
        Neurosdk2.set66HzFEnabled(this.guid, false);
    }

    getSupportedChannels(): Array<EEGChannelInfo>{
        return Neurosdk2.getSupportedChannels(this.guid);
    }

    getAmplifierParam(): BrainBit2AmplifierParam{
        return Neurosdk2.getAmplifierParamBB2(this.guid);
    }
    setAmplifierParam(param: BrainBit2AmplifierParam){
        Neurosdk2.setAmplifierParamBB2(this.guid, param);
    }

    getSamplingFrequencyMEMS(): SensorSamplingFrequency {
        return Neurosdk2.getSamplingFrequencyMEMS(this.guid);
    }

    getSamplingFrequencyFPG(): SensorSamplingFrequency {
        return Neurosdk2.getSamplingFrequencyFPG(this.guid);
    }

    getSamplingFrequencyResist(): SensorSamplingFrequency {
        return Neurosdk2.getSamplingFrequencyResist(this.guid);
    }

    getIrAmplitude(): IrAmplitude {
        return Neurosdk2.getIrAmplitude(this.guid);
    }
    setIrAmplitude(amp: IrAmplitude) {
        Neurosdk2.setIrAmplitude(this.guid, amp)
    }

    getRedAmplitude(): RedAmplitude {
        return Neurosdk2.getRedAmplitude(this.guid);
    }
    setRedAmplitude(amp: RedAmplitude) {
        Neurosdk2.setRedAmplitude(this.guid, amp);
    }

    getAmpMode(): SensorAmpMode {
        return Neurosdk2.getAmpMode(this.guid);
    }

    getAccSens(): SensorAccelerometerSensitivity {
        return Neurosdk2.getAccSens(this.guid);
    }
    setAccSens(accSens: SensorAccelerometerSensitivity) {
        Neurosdk2.setAccSens(this.guid, accSens);
    }

    getGyroSens(): SensorGyroscopeSensitivity {
        return Neurosdk2.getGyroSens(this.guid);
    }
    setGyroSens(gyroSens: SensorGyroscopeSensitivity) {
        Neurosdk2.setGyroSens(this.guid, gyroSens);
    }

    pingNeuroSmart(marker: number): Promise<string> {
        return Neurosdk2.pingNeuroSmart(this.guid, marker);
    }

    AddAMPModeChanged(callback: (AMPMode: SensorAmpMode) => void) {
        this.newAMPModeChanged = Neurosdk2.onAmpModeChanged((data: EventAmpModeChangedData) => { if (data.GUID === this.guid) callback(data.data) } );
        this.AMPModeChanged = this.nativeEventEmitter.addListener(
            Neurosdk2.AMPMODE_CHANGED_EVENT_NAME,
            (data) => { if (data.GUID === this.guid) callback(data.data) }
        );
    }
    RemoveAMPModeChanged() {
        if (this.AMPModeChanged != undefined){
            this.AMPModeChanged.remove();
            this.AMPModeChanged = undefined
        }
        if (this.newAMPModeChanged != undefined){
            this.newAMPModeChanged.remove();
            this.newAMPModeChanged = undefined
        }
    }

    AddMEMSDataChanged(callback: (data: Array<MEMSData>) => void) {
        this.newMEMSDataChanged = Neurosdk2.onMEMSReceived((data: EventNeuroSmartMEMSData) => { if (data.GUID === this.guid) callback(data.data) } );
        this.MEMSDataChanged = this.nativeEventEmitter.addListener(
            Neurosdk2.MEMSDATA_CHANGED_EVENT_NAME,
            (data) => { if (data.GUID === this.guid) callback(data.data) }
        );
    }
    RemoveMEMSDataChanged() {
        if (this.MEMSDataChanged != undefined){
            this.MEMSDataChanged.remove();
            this.MEMSDataChanged = undefined
        }
        if (this.newMEMSDataChanged != undefined){
            this.newMEMSDataChanged.remove();
            this.newMEMSDataChanged = undefined
        }   
    }

    AddNeuroSmartFPGDataChanged(callback: (FPGData: Array<FPGData>) => void) {
        this.newNeuroSmartFPGChanged = Neurosdk2.onNeuroSmartFPGReceived((data: EventFPGData) => { if (data.GUID === this.guid) callback(data.data) } );
        this.neuroSmartFPGChanged = this.nativeEventEmitter.addListener(
            Neurosdk2.NEUROSMART_FPGDATA_CHANGED_EVENT_NAME,
            (signalData) => { if (signalData.GUID === this.guid) callback(signalData.data) }
        );
    }
    RemoveNeuroSmartFPGDataChanged() {
        if (this.neuroSmartFPGChanged != undefined){
            this.neuroSmartFPGChanged.remove();
            this.neuroSmartFPGChanged = undefined;
        }
        if (this.newNeuroSmartFPGChanged != undefined){
            this.newNeuroSmartFPGChanged.remove();
            this.newNeuroSmartFPGChanged = undefined;
        } 
    }

    AddSignalReceived(callback: (signalData: Array<SignalChannelsData>) => void) {
        this.newSignalDataReceived = Neurosdk2.onBrainBit2SignalReceived((data: EventBrainBit2SignalData) => { if (data.GUID === this.guid) callback(data.data) } );
        this.signalDataReceived = this.nativeEventEmitter.addListener(
            Neurosdk2.BRAINBIT2_SIGNAL_RECEIVED_EVENT_NAME,
            (data) => { if (data.GUID === this.guid) callback(data.data) }
        );
    }
    RemoveSignalReceived() {
        if (this.signalDataReceived != undefined) {
            this.signalDataReceived.remove();
            this.signalDataReceived = undefined;
        }
          if (this.newSignalDataReceived != undefined) {
            this.newSignalDataReceived.remove();
            this.newSignalDataReceived = undefined;
        }  
    }
    AddResistanceReceived(callback: (resistanceData: Array<ResistRefChannelsData>) => void) {
        this.newResistDataReceived = Neurosdk2.onBrainBit2ResistReceived((data: EventBrainBit2ResistData) => { if (data.GUID === this.guid) callback(data.data) } );
        this.resistDataReceived = this.nativeEventEmitter.addListener(
            Neurosdk2.BRAINBIT2_RESIST_RECEIVED_EVENT_NAME,
            (data) => { if (data.GUID === this.guid) callback(data.data) }
        );
    }
    RemoveResistanceReceived() {
        if (this.resistDataReceived != undefined){
            this.resistDataReceived.remove();
            this.resistDataReceived = undefined;
        }
        if (this.newResistDataReceived != undefined){
            this.newResistDataReceived.remove();
            this.newResistDataReceived = undefined;
        }    
    }
}