import type { EmitterSubscription, EventSubscription } from "react-native";
import Sensor from "../Sensor";
import type { BrainBitResistData, BrainBitSignalData, SensorGain, EventBrainBitSignalData, EventBrainBitResistData } from "../NativeNeurosdk2";
import { Neurosdk2 } from "../ModuleResolver";

export default class BrainBitSensor extends Sensor {

    private resistDataReceived: EmitterSubscription | undefined;
    private newResistDataReceived: EventSubscription | undefined;
    private signalDataReceived: EmitterSubscription | undefined;
    private newSignalDataReceived: EventSubscription | undefined;

    constructor(guid: string) {
        super(guid)
    }

    enable66HzFilter(): void{
        Neurosdk2.set66HzFEnabled(this.guid, true);
    }

    disable66HzFilter(): void{
        Neurosdk2.set66HzFEnabled(this.guid, false);
    }

    AddSignalReceived(callback: (signalData: Array<BrainBitSignalData>) => void) {
        this.newSignalDataReceived = Neurosdk2.onBrainBitSignalReceived((data: EventBrainBitSignalData) => { if (data.GUID === this.guid) callback(data.data) } );
        this.signalDataReceived = this.nativeEventEmitter.addListener(
            Neurosdk2.BRAINBIT_SIGNAL_CHANGED_EVENT_NAME,
            (data) => { if (data.GUID === this.guid) callback(data.data) }
        );
    }
    RemoveSignalReceived() {
        if (this.signalDataReceived != undefined){
            this.signalDataReceived.remove();
            this.signalDataReceived = undefined;
        }
        if (this.newSignalDataReceived != undefined){
            this.newSignalDataReceived.remove();
            this.newSignalDataReceived = undefined;
        }    
    }
    AddResistanceReceived(callback: (resistanceData: BrainBitResistData) => void) {
        this.newResistDataReceived = Neurosdk2.onBrainBitResistReceived((data: EventBrainBitResistData) => { if (data.GUID === this.guid) callback(data.data) } );
        this.resistDataReceived = this.nativeEventEmitter.addListener(
            Neurosdk2.BRAINBIT_RESISTANCE_CHANGED_EVENT_NAME,
            (data) => { if (data.GUID === this.guid) callback(data.data) }
        );
    }
    RemoveResistanceReceived() {
        if (this.resistDataReceived != undefined) {
            this.resistDataReceived.remove();
            this.resistDataReceived = undefined;
        }
        if (this.newResistDataReceived != undefined) {
            this.newResistDataReceived.remove();
            this.newResistDataReceived = undefined;
        }    
    }

    getGain(): SensorGain {
        return Neurosdk2.getGain(this.guid);
    }

    setGain(gain: SensorGain){
      Neurosdk2.setGain(this.guid, gain);
    }

}